/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import android.content.Context;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.PopupMenu;
import android.widget.PopupMenu.OnMenuItemClickListener;

import net.micode.notes.R;

public class DropdownMenu {
    // 定义一个按钮，用于显示和触发下拉菜单
    private Button mButton;
    // 定义一个弹出菜单，用于显示菜单项
    private PopupMenu mPopupMenu;
    // 定义一个菜单类，用于存储菜单项
    private Menu mMenu;

    public DropdownMenu(Context context, Button button, int menuId) {
        mButton = button;
        // 设置背景为下拉图标
        mButton.setBackgroundResource(R.drawable.dropdown_icon);
        // 初始化弹出菜单，并绑定按钮对象
        mPopupMenu = new PopupMenu(context, mButton);
        // 获取菜单对象，加载菜单资源
        mMenu = mPopupMenu.getMenu();
        mPopupMenu.getMenuInflater().inflate(menuId, mMenu);
        // 设置点击监听器，点击时弹出菜单
        mButton.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                mPopupMenu.show();
            }
        });
    }

    // 用于设置下拉菜单点击监听器
    public void setOnDropdownMenuItemClickListener(OnMenuItemClickListener listener) {
        // 如果弹出菜单不为空则为其设置监听器
        if (mPopupMenu != null) {
            mPopupMenu.setOnMenuItemClickListener(listener);
        }
    }

    // 可根据id查找菜单项
    public MenuItem findItem(int id) {
        return mMenu.findItem(id);
    }

    // 用于设置按钮标题
    public void setTitle(CharSequence title) {
        mButton.setText(title);
    }
}
